package RSSReader::Gtk2UI::Category::MessageList;
use strict;
use RSSReader;
use RSSReader::Feed;
use RSSReader::Category;
use RSSReader::Feed::Message;
use RSSReader::Notifier;
use Data::Dumper;

use base qw(RSSReader::Notifier RSSReader::Logger RSSReader::Gtk2UI::MessageList);

our $_cat_msg_lists;

sub new {
	my $c = shift;
	my %a = @_;
	if (exists $_cat_msg_lists->{$a{id}}) { return $_cat_msg_lists->{$a{id}}; }
	
	my $s = $c->SUPER::new(%a);
	$s->{_cat} = RSSReader::Category->new(id=>$a{id});
	$s->{id}    = $a{id};

	RSSReader->new->notify_me(event=>'new_message', 	  	 caller=>$s, call=>'_new_message');
	RSSReader->new->notify_me(event=>'message_marked_unread',	 caller=>$s, call=>'_new_message');
	RSSReader->new->notify_me(event=>'feed_moved_after',		 caller=>$s, call=>'_feed_moved');

	$s->_populate();

	$_cat_msg_lists->{$a{id}} = $s;

    return $s;
}

sub _feed_moved {
    my $s = shift;
    my %a = @_;

    if ($a{from}->{id} eq $s->{id}) {
	foreach ( $a{feed}->messages() ) {
	  push @{$s->{_to_remove}}, $_;
	}
    }
    elsif ( $a{to}->{id} eq $s->{id}) {
	foreach ( $a{feed}->unread_messages() ) {
		$s->add_msg($_);
	}
    }

}

sub _new_message {
    my $s = shift;
    my %a = @_;
                                                                                                                                            
    return unless ($s->{_cat}->is_my_feed($a{message}->get_feed, recursive=>1));
    $s->SUPER::_new_message(@_);
}

sub _populate {
    my $s = shift;
    foreach my $msg (@{$s->{_cat}->unread_messages}){
	$s->add_msg($msg);	
    }
}

sub add_msg {
    my $s = shift;
    my $msg = shift;
    $msg->notify_me(event=>'mark_read', caller=>$s, call=>'_message_marked_read');
    $s->SUPER::add_msg($msg,@_);
}

sub _message_marked_read {
    my $s = shift;
    my %a = @_;
    my $msg = $a{where};
    return unless $msg;

    $s->log_debug("A message was marked read, should remove");

    push @{$s->{_to_remove}}, $msg;
}

sub show {
        my $s = shift;
	my %a = @_;
                                                                                
        $s->SUPER::show(%a);

        my $k = 1;
        while (my $col = $s->{_list_view}->get_column($k) ) {
                my $size;
                if ( $s->{_prefs}->per_view_columns() ) {
                        $size = $s->{_prefs}->get('category_view_'.$s->{_cat}->id(),"column_".$col->{_position}."_width");
                } else {
                        $size = $s->{_prefs}->get('category_view_global', "column_".$col->{_position}."_width");
                }
                $k++;
                $size = 50 unless $size;
                $s->log_spam("set col ".$col->{_position}." size $size");
                $col->set_fixed_width($size);
        }
}
                                                                                                                                                                            
sub hide {
        my $s = shift;
        my $k = 1;
        while (my $col = $s->{_list_view}->get_column($k) ) {
                $s->log_spam("saving col ".$col->{_position}." with size : ".$col->get_width);
                my $size = $col->get_width || 1;
                $s->{_prefs}->set('category_view_'.$s->{_cat}->id(),"column_".$col->{_position}."_width",$size );
                unless ( $s->{_prefs}->per_view_columns() ) {
                        $s->{_prefs}->set('category_view_global', "column_".$col->{_position}."_width",$size );
                }
                $k++;
        }
}

1;
