package RSSReader::Gtk2UI::Notifiers::TrayIcon;
use RSSReader;
use base qw(RSSReader::Gtk2UI::Notifiers RSSReader::Logger);
use strict;
use Data::Dumper;
use RSSReader::Gtk2UI::GeneralProperties;
use RSSReader::Gtk2UI;

our $_notifier_tray_icon;
our $_failed_load = undef;

sub display {
  my $s = shift;
  if ( $s->is_enabled() && $s->is_active() && (!$s->{widget})) {
  	$s->create_widget;
  }
}

sub new { 
  my $c = shift;

  return $_notifier_tray_icon if $_notifier_tray_icon;
 
  my $s = { };
  bless $s, $c;



  $_notifier_tray_icon = $s;
  return $_notifier_tray_icon;
}

#a message as arrived and i should notify that.
sub messages_arrived {
	my $s = shift;
	my %a = @_;
	
	return 1 unless ($s->is_active && $s->is_enabled);

	return 1 if $s->{is_notifying};

	$s->{is_notifying} = 1;

	$s->{event_box}->remove($s->{img});
	$s->{event_box}->add($s->{img_not});
	$s->{event_box}->show_all;

	return 1;
}

#user as activated syndigator
sub messages_noticed {
	my $s = shift;
	my %a = @_;
	
	return 1 unless ($s->is_active && $s->is_enabled);

	return 1 unless $s->{is_notifying};

	$s->{is_notifying} = 0;

	$s->{event_box}->remove($s->{img_not});
	$s->{event_box}->add($s->{img});
	$s->{event_box}->show_all;

	return 1;
}

sub create_widget {
	my $s = shift;
  	RSSReader::Gtk2UI::IconFactory->add_icon( name=>"trayicon",
						  label=>"Trayicon",
						  file =>"trayicon.gif",
						);
  	RSSReader::Gtk2UI::IconFactory->add_icon( name=>"trayicon-notifying",
						  label=>"Trayicon notifying",
						  file =>"trayicon-notify.gif",
						);

	$s->{widget} = Gtk2::TrayIcon->new("Syndigator");

	$s->{event_box} = Gtk2::EventBox->new();
	$s->{img}     = Gtk2::Image->new_from_stock("syndigator-trayicon",          'small-toolbar');
	$s->{img_not} = Gtk2::Image->new_from_stock("syndigator-trayicon-notifying",'small-toolbar'); 
	$s->{event_box}->add($s->{img});
	$s->{widget}->add( $s->{event_box} );
	$s->{widget}->show_all;

	my $m_w = RSSReader::Gtk2UI::MainWindow->new();

	RSSReader->new()->notify_me(event =>'new_message', caller=>$s, call=>'messages_arrived');

	$m_w->notify_me(event => 'view_changed',     caller=>$s, call=>'messages_noticed');	
	$m_w->notify_me(event => 'message_selected', caller=>$s, call=>'messages_noticed');	

	$s->{event_box}->signal_connect('button-press-event', sub { $s->docklet_clicked(@_) });

}

sub pop_up_options {
	my $s = shift;
	my @items = (
	     [
                '/Fetch all feeds',
                "",
                sub { RSSReader::Gtk2UI->new()->get_feeds() },
                1,
		'<StockItem>',
                'gtk-refresh'
             ],
	     [
                '/Preferences',
                undef,
                sub { RSSReader::Gtk2UI::GeneralProperties->run() },
                1,
		'<StockItem>',
                'gtk-preferences'
             ],
	     [
                '/Exit',
                undef,
                sub { RSSReader::Gtk2UI->new()->_exit },
                1,
		'<StockItem>',
                'gtk-quit'
             ],
	);

	my $f = Gtk2::ItemFactory->new('Gtk2::Menu', 
				        '<trayiconpopup>', undef);
    	$f->create_items(1, @items);
	my $menu = $f->get_widget('<trayiconpopup>');
    	$menu->popup(undef, undef, undef,undef, 0, Gtk2->get_current_event_time());
}

sub docklet_clicked {
	my $s = shift;
	my ($event_box, $event) = @_;
	my $m_w = RSSReader::Gtk2UI::MainWindow->new;
	
	if ($event->button == 3) {
	#it is a right click;
		$s->pop_up_options;
		return 1;
	}

	if ($m_w->widget->visible) {
		$m_w->hide;
	} else {
		$m_w->show;
		$s->messages_noticed;
	}
}

sub destroy_widget {
	my $s = shift;
	$s->{widget}->destroy;
	delete $s->{widget};
}

sub enable {
	my $s = shift;
	$s->SUPER::enable(@_);
	unless ( $s->{widget} ) {
	  $s->create_widget;
	}
}

sub disable {
	my $s = shift;
	$s->SUPER::disable(@_);
	if ( $s->{widget} ) {
	  $s->destroy_widget;
	}
	my $window = RSSReader::Gtk2UI::MainWindow->new();
	$window->show unless $window->widget()->visible; 
}

sub is_active {
   my $c = shift;	
   return $_failed_load ? 0 : 1 ;
}

sub fail_reason {
   my $c = shift;
   return $_failed_load;
}

sub id { return "notifier_trayicon"; }

sub name { return "Tray Icon"; }

sub version { return '0.01' }

sub config_options { 
     my $s = shift;
     return [ ];
}

sub description {
    return "Displays a icon in the notification area that will notify the user of new messages";
}

eval { require Gtk2::TrayIcon };
if ( $@ ) {
   $_failed_load = "Gtk2::TrayIcon is needed for this notifier";
}
 
RSSReader::Gtk2UI::Notifiers->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
