package RSSReader::Importer::BottomFeeder;

use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Importer;
use Data::Dumper;
use base qw( RSSReader::Logger RSSReader::Importer );


sub new {
  my $c = shift;
  my %a = @_;
  my $bf_file = $a{file} || $ENV{HOME}.'/btfOPMLExport.xml';

  return bless { bf_file => $bf_file}, $c
}

sub fetch {
  my $s = shift;
  my %a = @_;

  my @feeds;

  $s->log_spam("Importing BottomFeeder RDFs...");

  my $bf_file = $a{file};

  unless (open(FH, '<', $bf_file)) {
    $s->log_warn("Can't open BottomFeeder export file: $!...");
    $@ = "Could not find BottomFeeder export file";
    return undef;
  }
  my $import ='';
  local $_;
  while (<FH>) { 
    $import.=$_ ;
  }             
  close(FH);

  my @imports = split( /<outline/ , $import);
  foreach my $outline (@imports){
     my ($url) = $outline =~ /xmlUrl="(.*?)"/;
     ($url) = $outline =~ /htmlUrl="(.*?)"/ unless $url; #Old version...
     my ($label) = $outline =~ /text="(.*?)"/;
     next unless ($url && $label);
     push @feeds, { 
                     label          => $label,
                     url            => $url
                   };
     
  }
  $s->log_spam("...Fetched\n");
    my %to_return = (
                    feeds      => \@feeds,
                    categories => []
                  );

  return wantarray ? %to_return : \%to_return;
}


sub id { return "importer_bottomfeeder"; }

sub name { return "BottomFeeder"; }

sub config_options {
  my $s=shift; 
  return [{
            default => $s->{bf_file}, 
            name    => 'file', 
            desc    => 'BottomFeeder export file', 
            type    => 'file'
         }];
}

sub description {
	return 'Import your BottomFeeder RSS Feeds';
}

RSSReader::Importer->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
